const path = require('path');
const fs = require('fs');
const chalk = require('chalk');
const debug = require('./utils/debugPlus')('index');
const register = require('./register');

const CWD = process.cwd();

/**
 * 如果是使用nodejs参数"-r"或"--require"引入这个文件则自动注册编译器
 * @see {@link https://nodejs.org/docs/latest/api/cli.html#cli_r_require_module}
 * @see {@link https://nodejs.org/docs/latest/api/cli.html#cli_node_options_options}
 */
function autoRegister(checkFileName = __filename) {
  const cwdModulesPath = path.resolve(CWD, 'node_modules');
  const nodeEnvOptionsArgs = process.env.NODE_OPTIONS ? process.env.NODE_OPTIONS.split(/ +/) : [];
  const preLoadFiles = process.execArgv
    .concat(nodeEnvOptionsArgs)
    .filter(function(val, index, ary) {
      return index !== 0 && /^(-r|--require)$/.test(ary[index - 1]);
    })
    .map(v => require.resolve(v, { paths: [CWD, cwdModulesPath] }));
  if (preLoadFiles.includes(checkFileName)) {
    debug(chalk.bgRed.whiteBright(' Auto start register!! '));
    register();
  } else if (preLoadFiles.length >= 1) {
    debug(chalk.bgRed.whiteBright('Non automatic start register!'));
    debug('checkFileName: %s', checkFileName);
    debug('preLoadFiles: %O', preLoadFiles);
  }
}

autoRegister(__filename);

module.exports = register;
