/**
 * 根据 tyepscript 的配置(baseUrl和paths)生成 babel-plguin-module-resolve的配置
 */
const path = require('path');
const debug = require('./debugPlus')('generate resolver config');

const extensions = ['.tsx', '.ts', '.jsx', '.js'];

/** 排除声明文件目录 */
const IGNORE_TYPES_DIRECTORY = /(?:^|[\\/])([@#$%^&*]?type|typing|declaration)s?[\\/]/i;

function gen(tsConfig = {}, configFilePath) {
  const { baseUrl, paths } = tsConfig.compilerOptions || {};
  if (baseUrl && paths) {
    const alias = {};
    for (let key in paths) {
      const val = paths[key];
      if (IGNORE_TYPES_DIRECTORY.test(val[0])) {
        debug('Ignore types directory "%s"', val[0]);
        continue;
      }
      if (key === '*') {
        continue;
        debug('Ignore "*"');
      }
      const res = key.match(/^(.+?)\/\*/);
      if (res) {
        alias[res[1]] = val[0].replace(/^(?:\.\/)*(.*?)\/?\*$/, './$1');
      } else if (/^[^\s\\/]+$/.test(key)) {
        alias[key] = val[0].replace(/^(?:.\/)*(.+?)(?:\.(\w+))?$/, './$1');
      } else {
        debug('not support: %o', { [key]: val });
      }
    }
    const config = { alias, cwd: path.resolve(path.dirname(configFilePath), baseUrl) };
    debug('ResolverConfig: %O', config);
    return config;
  }
  return {
    extensions
  };
}

module.exports = gen;
