const fs = require('fs');
const debug = require('./debugPlus')('getHelpfulFrame');
const sourceMapSupport = require('source-map-support');

//克隆传入的frame并使用sourcemap解释出正确的文件名、行号和列号
const wrapCallSite = sourceMapSupport.wrapCallSite;


const excludeFileList = ['module.js', 'internal/module.js', 'event.js'];

/**
 * 从错误信息栈中获取有用的(并且使用sourcemap取回真实信息的)帧
 */
module.exports = function getHelpfulFrame(stack, multiple = false) {
  const result = stack[multiple ? 'filter' : 'find'](function(frame) {
    const fileName = frame.getFileName();
    if (fileName && !/node_modules/.test(fileName) && !excludeFileList.includes(fileName)) {
      if (fs.existsSync(fileName)) {
        return true;
      }
      // debug('%s is not exist.', fileName);
    }
  });
  if (!result) return null;
  return Array.isArray(result) ? result.map(v => wrapCallSite(v)) : wrapCallSite(result);
};
