const openEditor = require('open-editor');
const envEditor = require('env-editor');
const chalk = require('chalk');
const childProcess = require('child_process');
const debug = require('./debugPlus')('openEditor');
const inquirer = require('inquirer');
const repl = require('repl');

const DEFAULT_OPTIONS = {
  editor: 'code',
  delay: 1000, //额外的
  confirm: true, //额外的
  timeout: 8e3 //额外的
};
const DEFAULT_QUESTIONS = {
  type: 'confirm',
  message: 'Open file(s) with editor?',
  name: 'result',
  default: true
};

const pDelay = function(time = 1, val) {
  let timer;
  const pe = new Promise(function(re) {
    timer = setTimeout(re, time, val);
  });
  pe.abort = function() {
    clearTimeout(timer);
  };
  return pe;
};

const KEY_NOT_MERGE = Symbol('not_merge');

/**
 * 复制https://github.com/sindresorhus/open-editor/blob/master/index.js#L56，修复window无法打开的问题
 */
function open(files, opts) {
  if (typeof opts.delay === 'number' && opts.delay >= 1) {
    const { delay, ...otherOpts } = opts;
    return pDelay(delay).then(() => {
      open(files, otherOpts);
    });
  }
  const result = openEditor.make(files, opts);
  if (opts.confirm && opts.timeout) {
    const { timeout, confirm, ...otherOpts } = opts;
    return showConfirmPrompt(`Open file(s) with editor(${chalk.blueBright(envEditor.get(result.bin).name)})?`, timeout).then(confirm => {
      confirm && open(files, otherOpts);
      return confirm;
    });
  }

  const stdio = result.isTerminalEditor ? 'inherit' : 'ignore';
  const cp =
    process.platform === 'win32'
      ? childProcess.exec(`${result.bin} ${result.args.join(' ')}`)
      : childProcess.spawn(result.bin, result.args, {
          detached: true,
          stdio
        });
  // Fallback
  cp.on('error', () => {
    const result = openEditor.make(files, Object.assign({}, opts, { editor: '' }));

    for (const file of result.args) {
      opn(file, { wait: false });
    }
  });
  if (result.isTerminalEditor) {
    cp.on('exit', process.exit);
  } else {
    cp.unref();
  }
}

function make(files, options) {
  const opts = { ...DEFAULT_OPTIONS, ...options };
  return openEditor.make(files, opts);
}
function showConfirmPrompt(message, timeout = 1333) {
  const questions = { ...DEFAULT_QUESTIONS, message: message || DEFAULT_QUESTIONS.message };
  const pmt = inquirer.prompt(questions);
  const timeoutPromise = pDelay(timeout);
  const timeoutClose = timeoutPromise.then(function() {
    console.log(chalk.cyan('Timeout'));
    pmt.ui.close();
    return false;
  });
  return Promise.race([
    pmt.then(v => {
      timeoutPromise.abort();
      return v[DEFAULT_QUESTIONS.name];
    }),
    timeoutClose
  ]);
}

module.exports = function openEditor(files, options) {
  const opts = { ...DEFAULT_OPTIONS, ...options };
  debug('openFiles: %O', files);
  debug('options: %O',opts);
  return open(files, opts);
};
module.exports.make = make;
module.exports.showConfirmPrompt;
module.exports.DEFAULT_CONFIG = DEFAULT_OPTIONS;
