const chalk = require('chalk');
const debug = require('./debugPlus')('repairStackFilename');
const retrieveFilePath = require('./retrieveFilePath');

function retrieveFilePathFromStackText(opts, text) {
  return text.replace(/\((.+):(\d+):(\d+)\)/g, function(matchStr, p1, p2, p3) {
    return `(${retrieveFilePath(opts, p1)}:${p2}:${p3})`;
  });
}

/**
 * 还原栈里每一帧的文件路径(已被伪造的)
 */
module.exports = function repairStackFilename(opts, stack) {
  const { sourceMapRootPathAlias, rootPathAlias } = opts;
  return stack.map(function(frame) {
    const mockFileName = frame.getFileName();
    if (mockFileName && (mockFileName.startsWith(sourceMapRootPathAlias) || mockFileName.startsWith(rootPathAlias))) {
      debug(chalk.blueBright('@mockFileName'), mockFileName);
      Object.defineProperty(frame, 'getFileName', { value: () => retrieveFilePath(opts, mockFileName) });
      frame._toString = frame.toString;
      Object.defineProperty(frame, 'toString', {
        value: function() {
          return retrieveFilePathFromStackText(opts, this._toString());
        }
      });
      debug(chalk.yellow('#realFilename:'), frame.getFileName());
    }
    return frame;
  });
};
